/*==============================================================*/
/* Nom de SGBD :  Microsoft SQL Server 2008                     */
/* Date de cration :  05/07/2012 10:54:05                      */
/*==============================================================*/


/*==============================================================*/
/* Table : eventtypes                                           */
/*==============================================================*/
create table eventtypes (
   typeid               int                  not null,
   name                 nvarchar(255)        not null,
   iconname             nvarchar(255)        null,
   exposedstring        nvarchar(2048)       null,
   constraint pk_eventtypes primary key (typeid)
)
go

if exists(select 1 from sys.extended_properties p where
      p.major_id = object_id('eventtypes')
  and p.minor_id = (select c.column_id from sys.columns c where c.object_id = p.major_id and c.name = 'exposedstring')
)
begin
   declare @currentuser sysname
select @currentuser = user_name()
execute sp_dropextendedproperty 'MS_Description', 
   'user', @currentuser, 'table', 'eventtypes', 'column', 'exposedstring'

end


select @currentuser = user_name()
execute sp_addextendedproperty 'MS_Description', 
   'String.Format function based to expose event on wall
   {0} = Author
   {1} = Object',
   'user', @currentuser, 'table', 'eventtypes', 'column', 'exposedstring'
go

/*==============================================================*/
/* Table : followedusers                                        */
/*==============================================================*/
create table followedusers (
   userid               int                  not null,
   followeduserid       int                  not null,
   since                smalldatetime        null,
   constraint pk_followedusers primary key (userid, followeduserid)
)
go

/*==============================================================*/
/* Index : fk_users_followed                                    */
/*==============================================================*/
create index fk_users_followed on followedusers (
userid asc
)
go

/*==============================================================*/
/* Index : fk_followed_users                                    */
/*==============================================================*/
create index fk_followed_users on followedusers (
followeduserid asc
)
go

/*==============================================================*/
/* Table : hotworkitems                                         */
/*==============================================================*/
create table hotworkitems (
   id                   int                  not null,
   userid               int                  not null,
   projectid            int                  not null,
   since                smalldatetime        not null default getdate(),
   constraint pk_hotworkitems primary key (id)
)
go

if exists(select 1 from sys.extended_properties p where
      p.major_id = object_id('hotworkitems')
  and p.minor_id = (select c.column_id from sys.columns c where c.object_id = p.major_id and c.name = 'id')
)
begin
   declare @currentuser sysname
select @currentuser = user_name()
execute sp_dropextendedproperty 'MS_Description', 
   'user', @currentuser, 'table', 'hotworkitems', 'column', 'id'

end


select @currentuser = user_name()
execute sp_addextendedproperty 'MS_Description', 
   'ID shared with TFS',
   'user', @currentuser, 'table', 'hotworkitems', 'column', 'id'
go

/*==============================================================*/
/* Index : fk_hotworki_users                                    */
/*==============================================================*/
create index fk_hotworki_users on hotworkitems (
userid asc
)
go

/*==============================================================*/
/* Index : fk_hotworki_projects                                 */
/*==============================================================*/
create index fk_hotworki_projects on hotworkitems (
projectid asc
)
go

/*==============================================================*/
/* Table : projectprofiles                                      */
/*==============================================================*/
create table projectprofiles (
   projectid            int                  not null,
   ppkey                varchar(32)          not null,
   ppstringvalue        ntext                null,
   ppbinaryvalue        image                null,
   constraint pk_projectprofiles primary key (projectid, ppkey)
)
go

if exists (select 1 from  sys.extended_properties
           where major_id = object_id('projectprofiles') and minor_id = 0)
begin 
   declare @currentuser sysname 
select @currentuser = user_name() 
execute sp_dropextendedproperty 'MS_Description',  
   'user', @currentuser, 'table', 'projectprofiles' 
 
end 


select @currentuser = user_name() 
execute sp_addextendedproperty 'MS_Description',  
   'Other parameters for project', 
   'user', @currentuser, 'table', 'projectprofiles'
go

/*==============================================================*/
/* Index : fk_projectp_projects                                 */
/*==============================================================*/
create index fk_projectp_projects on projectprofiles (
projectid asc
)
go

/*==============================================================*/
/* Table : projects                                             */
/*==============================================================*/
create table projects (
   projectid            int                  not null,
   name                 nvarchar(512)        not null,
   url                  varchar(512)         null,
   currentsprint        nvarchar(1024)       null,
   subscribecheckin     int                  null,
   tokencheckin         uniqueidentifier     null,
   subscribewichanged   int                  null,
   tokenwichanged       uniqueidentifier     null,
   notuse               bit                  not null,
   constraint pk_projects primary key (projectid)
)
go

if exists (select 1 from  sys.extended_properties
           where major_id = object_id('projects') and minor_id = 0)
begin 
   declare @currentuser sysname 
select @currentuser = user_name() 
execute sp_dropextendedproperty 'MS_Description',  
   'user', @currentuser, 'table', 'projects' 
 
end 


select @currentuser = user_name() 
execute sp_addextendedproperty 'MS_Description',  
   'Project List', 
   'user', @currentuser, 'table', 'projects'
go

if exists(select 1 from sys.extended_properties p where
      p.major_id = object_id('projects')
  and p.minor_id = (select c.column_id from sys.columns c where c.object_id = p.major_id and c.name = 'subscribecheckin')
)
begin
   declare @currentuser sysname
select @currentuser = user_name()
execute sp_dropextendedproperty 'MS_Description', 
   'user', @currentuser, 'table', 'projects', 'column', 'subscribecheckin'

end


select @currentuser = user_name()
execute sp_addextendedproperty 'MS_Description', 
   'Checkin event Subscribe ID',
   'user', @currentuser, 'table', 'projects', 'column', 'subscribecheckin'
go

if exists(select 1 from sys.extended_properties p where
      p.major_id = object_id('projects')
  and p.minor_id = (select c.column_id from sys.columns c where c.object_id = p.major_id and c.name = 'subscribewichanged')
)
begin
   declare @currentuser sysname
select @currentuser = user_name()
execute sp_dropextendedproperty 'MS_Description', 
   'user', @currentuser, 'table', 'projects', 'column', 'subscribewichanged'

end


select @currentuser = user_name()
execute sp_addextendedproperty 'MS_Description', 
   'WorkItemChanged event Subscribe ID',
   'user', @currentuser, 'table', 'projects', 'column', 'subscribewichanged'
go

/*==============================================================*/
/* Table : scrumpilotparams                                     */
/*==============================================================*/
create table scrumpilotparams (
   id                   uniqueidentifier     not null default newid(),
   buglevelok           int                  null,
   buglevelorange       int                  null,
   buglevelred          int                  null,
   codereviewlevelok    int                  null,
   codereviewlevelorange int                  null,
   codereviewlevelred   int                  null,
   testtodolevelok      int                  null,
   testtodolevelorange  int                  null,
   testtodolevelred     int                  null,
   kanbanmaxdoing       int                  null,
   kanbanmaxtotest      int                  null,
   yammerconsumer_key   varchar(256)         null,
   yammerconsumer_secret varchar(256)         null,
   constraint pk_scrumpilotparams primary key (id)
)
go

/*==============================================================*/
/* Table : speventcomments                                      */
/*==============================================================*/
create table speventcomments (
   eventcommentid       bigint               identity,
   eventid              bigint               null,
   userid               int                  null,
   comments             nvarchar(max)        not null,
   createdate           smalldatetime        null,
   constraint pk_speventcomments primary key (eventcommentid)
)
go

/*==============================================================*/
/* Index : fk_comments_spevents                                 */
/*==============================================================*/
create index fk_comments_spevents on speventcomments (
eventid asc
)
go

/*==============================================================*/
/* Index : fk_comments_users                                    */
/*==============================================================*/
create index fk_comments_users on speventcomments (
userid asc
)
go

/*==============================================================*/
/* Table : spevents                                             */
/*==============================================================*/
create table spevents (
   eventid              bigint               identity,
   typeid               int                  not null,
   userid               int                  null,
   projectid            int                  null,
   eventobject          xml                  null,
   createdate           smalldatetime        not null,
   comments             nvarchar(max)        null,
   constraint pk_spevents primary key (eventid)
)
go

if exists(select 1 from sys.extended_properties p where
      p.major_id = object_id('spevents')
  and p.minor_id = (select c.column_id from sys.columns c where c.object_id = p.major_id and c.name = 'eventobject')
)
begin
   declare @currentuser sysname
select @currentuser = user_name()
execute sp_dropextendedproperty 'MS_Description', 
   'user', @currentuser, 'table', 'spevents', 'column', 'eventobject'

end


select @currentuser = user_name()
execute sp_addextendedproperty 'MS_Description', 
   'Contain the Event Object from TFS',
   'user', @currentuser, 'table', 'spevents', 'column', 'eventobject'
go

/*==============================================================*/
/* Index : idate                                                */
/*==============================================================*/
create index idate on spevents (
createdate asc
)
go

/*==============================================================*/
/* Index : fk_events_eventtyp                                   */
/*==============================================================*/
create index fk_events_eventtyp on spevents (
typeid asc
)
go

/*==============================================================*/
/* Index : fk_events_users                                      */
/*==============================================================*/
create index fk_events_users on spevents (
userid asc
)
go

/*==============================================================*/
/* Index : fk_spevents_projects                                 */
/*==============================================================*/
create index fk_spevents_projects on spevents (
projectid asc
)
go

/*==============================================================*/
/* Table : sprintprojectiterations                              */
/*==============================================================*/
create table sprintprojectiterations (
   sprintid             int                  not null,
   projectid            int                  not null,
   currentsprintid      int                  not null,
   currentsprint        nvarchar(1024)       not null,
   constraint pk_sprintprojectiterations primary key (sprintid, projectid)
)
go

/*==============================================================*/
/* Index : fk_sprintpr_sprints                                  */
/*==============================================================*/
create index fk_sprintpr_sprints on sprintprojectiterations (
sprintid asc
)
go

/*==============================================================*/
/* Index : fk_sprintpr_projects                                 */
/*==============================================================*/
create index fk_sprintpr_projects on sprintprojectiterations (
projectid asc
)
go

/*==============================================================*/
/* Table : sprints                                              */
/*==============================================================*/
create table sprints (
   sprintid             int                  identity,
   name                 nvarchar(1024)       null,
   startdate            smalldatetime        null,
   enddate              smalldatetime        null,
   description          nvarchar(max)        null,
   retrospective        nvarchar(max)        null,
   state                tinyint              not null default 0
      constraint ckc_state_sprints check (state in (0,1,2,3)),
   constraint pk_sprints primary key (sprintid)
)
go

/*==============================================================*/
/* Table : tfsserverparams                                      */
/*==============================================================*/
create table tfsserverparams (
   id                   smallint             identity,
   tfsurl               varchar(512)         not null,
   tfsport              varchar(5)           not null,
   tfsdirectory         nvarchar(256)        null,
   tfsuser              nvarchar(512)        not null,
   tfspassword          nvarchar(512)        not null,
   tfsrealurl           varchar(512)         null,
   spwsaddress          varchar(512)         null,
   constraint pk_tfsserverparams primary key (id)
)
go

if exists (select 1 from  sys.extended_properties
           where major_id = object_id('tfsserverparams') and minor_id = 0)
begin 
   declare @currentuser sysname 
select @currentuser = user_name() 
execute sp_dropextendedproperty 'MS_Description',  
   'user', @currentuser, 'table', 'tfsserverparams' 
 
end 


select @currentuser = user_name() 
execute sp_addextendedproperty 'MS_Description',  
   'Parameters for connection to TFS', 
   'user', @currentuser, 'table', 'tfsserverparams'
go

if exists(select 1 from sys.extended_properties p where
      p.major_id = object_id('tfsserverparams')
  and p.minor_id = (select c.column_id from sys.columns c where c.object_id = p.major_id and c.name = 'spwsaddress')
)
begin
   declare @currentuser sysname
select @currentuser = user_name()
execute sp_dropextendedproperty 'MS_Description', 
   'user', @currentuser, 'table', 'tfsserverparams', 'column', 'spwsaddress'

end


select @currentuser = user_name()
execute sp_addextendedproperty 'MS_Description', 
   'SOAP address of Scrum Pilot server for Even posting from TFS',
   'user', @currentuser, 'table', 'tfsserverparams', 'column', 'spwsaddress'
go

/*==============================================================*/
/* Table : userimages                                           */
/*==============================================================*/
create table userimages (
   userid               int                  not null,
   contenttype          varchar(128)         null,
   img16                image                null,
   img32                image                null,
   img64                image                null,
   img96                image                null,
   constraint pk_userimages primary key (userid)
)
go

if exists(select 1 from sys.extended_properties p where
      p.major_id = object_id('userimages')
  and p.minor_id = (select c.column_id from sys.columns c where c.object_id = p.major_id and c.name = 'img16')
)
begin
   declare @currentuser sysname
select @currentuser = user_name()
execute sp_dropextendedproperty 'MS_Description', 
   'user', @currentuser, 'table', 'userimages', 'column', 'img16'

end


select @currentuser = user_name()
execute sp_addextendedproperty 'MS_Description', 
   'Image 16 x16',
   'user', @currentuser, 'table', 'userimages', 'column', 'img16'
go

/*==============================================================*/
/* Index : fk_userimag_users                                    */
/*==============================================================*/
create index fk_userimag_users on userimages (
userid asc
)
go

/*==============================================================*/
/* Table : userprofiles                                         */
/*==============================================================*/
create table userprofiles (
   userid               int                  not null,
   upkey                varchar(32)          not null,
   upstringvalue        ntext                null,
   upbinaryvalue        image                null,
   constraint pk_userprofiles primary key (userid, upkey)
)
go

/*==============================================================*/
/* Index : fk_userprof_users                                    */
/*==============================================================*/
create index fk_userprof_users on userprofiles (
userid asc
)
go

/*==============================================================*/
/* Table : users                                                */
/*==============================================================*/
create table users (
   userid               int                  identity,
   name                 nvarchar(512)        not null,
   mail                 varchar(256)         null,
   tfsuserid            varchar(512)         null,
   tfsname              nvarchar(512)        null,
   deleted              bit                  not null default 0,
   isadmin              bit                  not null default 0,
   authorizemail        bit                  not null default 1,
   constraint pk_users primary key (userid)
)
go

/*==============================================================*/
/* Index : itfs                                                 */
/*==============================================================*/
create index itfs on users (
tfsuserid asc
)
go

/*==============================================================*/
/* Index : itfsname                                             */
/*==============================================================*/
create index itfsname on users (
tfsname asc
)
go

alter table followedusers
   add constraint fk_followed_users foreign key (followeduserid)
      references users (userid)
go

alter table followedusers
   add constraint fk_users_followed foreign key (userid)
      references users (userid)
         on delete cascade
go

alter table hotworkitems
   add constraint fk_hotworki_projects foreign key (projectid)
      references projects (projectid)
go

alter table hotworkitems
   add constraint fk_hotworki_users foreign key (userid)
      references users (userid)
go

alter table projectprofiles
   add constraint fk_projectp_projects foreign key (projectid)
      references projects (projectid)
         on delete cascade
go

alter table speventcomments
   add constraint fk_comments_spevents foreign key (eventid)
      references spevents (eventid)
         on delete cascade
go

alter table speventcomments
   add constraint fk_comments_users foreign key (userid)
      references users (userid)
         on delete cascade
go

alter table spevents
   add constraint fk_events_eventtyp foreign key (typeid)
      references eventtypes (typeid)
go

alter table spevents
   add constraint fk_events_users foreign key (userid)
      references users (userid)
         on delete set null
go

alter table spevents
   add constraint fk_spevents_projects foreign key (projectid)
      references projects (projectid)
         on delete set null
go

alter table sprintprojectiterations
   add constraint fk_sprintpr_projects foreign key (projectid)
      references projects (projectid)
         on delete cascade
go

alter table sprintprojectiterations
   add constraint fk_sprintpr_sprints foreign key (sprintid)
      references sprints (sprintid)
         on delete cascade
go

alter table userimages
   add constraint fk_userimag_users foreign key (userid)
      references users (userid)
         on delete cascade
go

alter table userprofiles
   add constraint fk_userprof_users foreign key (userid)
      references users (userid)
         on delete cascade
go

