using System;
using System.Collections;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using Microsoft.TeamFoundation.WorkItemTracking.Client;

namespace ScrumPilot.TFSHelpers
{
    public class TFSQuery
    {
        private readonly WorkItemStore _wis;
        const string MainQuery = "SELECT * FROM WorkItems WHERE {0}";

        /*const string MainQuery = "SELECT  [System.Id], [System.WorkItemType],[System.State], [System.AssignedTo], [System.Title]," +
               "[Microsoft.VSTS.Common.StackRank],[Microsoft.VSTS.Scheduling.StartDate],[Microsoft.VSTS.Scheduling.FinishDate], " +
               "[Microsoft.VSTS.Scheduling.RemainingWork],[Microsoft.VSTS.TCM.ReproSteps]," +
               "[Microsoft.VSTS.CMMI.Blocked], [System.Reason],[Microsoft.VSTS.Common.ResolvedDate] , [Microsoft.VSTS.Common.Activity]," +
               "[ScrumPilot.Score.Level], [ScrumPilot.Score.Level],  [ScrumPilot.Score.Precedence],  [ScrumPilot.Score.Duration]," +
                "[ScrumPilot.Score.ScoringDate] FROM WorkItems WHERE {0}";*/


        public TFSQuery(WorkItemStore w)
        {
            this._wis = w;
        }

        public List<WorkItem> GetAllWorkItems()
        {
            const string sQuery = "select * FROM WorkItems  where [System.State] <> 'Removed'";

            return this.GetWorkItemList(sQuery);
        }

        public List<WorkItem> GetWorkItemList(string sQuery)
        {
            WorkItemCollection retVal = this.GetWorkItemCollection(sQuery);
            var lst = new List<WorkItem>(retVal.Count);
            lst.AddRange(retVal.Cast<WorkItem>());
            return lst;
        }

        public List<WorkItem> GetWorkItemList(IEnumerable<Tuple<string, string>> projects, List<string> types, List<WorkItemState> states = null, Boolean isTask = false, Boolean bProjectOnly = true)
        {
            String whereCond = this.AddTuplesProjectsCondition(projects, bProjectOnly);
            whereCond += this.AddStatesCondition(states, isTask);
            whereCond += this.AddTypesCondition(types);

            if (whereCond.StartsWith(" AND ("))
                whereCond = whereCond.Substring(4);

            WorkItemCollection retVal = this.GetWorkItemCollection(String.Format(MainQuery, whereCond));
            if (retVal != null)
            {
                var lst = new List<WorkItem>(retVal.Count);
                lst.AddRange(retVal.Cast<WorkItem>());
                return lst;
            }
            return null;
        }

        public List<WorkItem> GetWorkItemList(IDictionary<string, string> projects, List<string> types, List<WorkItemState> states = null, Boolean isTask = false, Boolean bProjectOnly = true)
        {
            var myPrjs = projects.Select(project => new Tuple<string, string>(project.Key, project.Value)).ToArray();
            return this.GetWorkItemList(myPrjs, types, states, isTask, bProjectOnly);
        }

        public List<WorkItem> GetWorkItemList(List<String> projects, List<WorkItemState> states, List<String> types, Boolean isTask)
        {
            String whereCond = this.AddProjectsCondition(projects);
            whereCond += this.AddStatesCondition(states, isTask);
            whereCond += this.AddTypesCondition(types);

            WorkItemCollection retVal = this.GetWorkItemCollection(String.Format(MainQuery, whereCond));
            var lst = new List<WorkItem>();
            if (retVal != null)
                lst.AddRange(retVal.Cast<WorkItem>());
            return lst;
        }

        public List<WorkItem> GetWorkItemList(String iteration, List<WorkItemState> states, List<String> types, Boolean isTask)
        {
            String whereCond = String.Format("[System.IterationPath] ='{0}'", iteration);
            whereCond += this.AddStatesCondition(states, isTask);
            whereCond += this.AddTypesCondition(types);

            WorkItemCollection retVal = this.GetWorkItemCollection(String.Format(MainQuery, whereCond));
            var lst = new List<WorkItem>();
            if (retVal != null)
                lst.AddRange(retVal.Cast<WorkItem>());
            return lst;
        }

        public List<WorkItem> GetWorkItemList(String iteration, List<String> types, Boolean isTask)
        {
            String whereCond = String.Format("[System.IterationPath] ='{0}'", iteration);
            whereCond += this.AddTypesCondition(types);

            WorkItemCollection retVal = this.GetWorkItemCollection(String.Format(MainQuery, whereCond));
            var lst = new List<WorkItem>();
            if (retVal != null)
                lst.AddRange(retVal.Cast<WorkItem>());
            return lst;
        }

        public List<WorkItem> GetWorkItemList(List<int> ids, List<String> types, Boolean isTask)
        {
            String whereCond = this.AddIdsCondition(ids);
            whereCond += this.AddTypesCondition(types);

            WorkItemCollection retVal = this.GetWorkItemCollection(String.Format(MainQuery, whereCond));
            var lst = new List<WorkItem>();
            if (retVal != null)
                lst.AddRange(retVal.Cast<WorkItem>());
            return lst;
        }

        public List<WorkItem> GetWorkItemList(List<int> ids, List<String> types, List<WorkItemState> states, Boolean isTask)
        {
            String whereCond = this.AddIdsCondition(ids);
            whereCond += this.AddStatesCondition(states, isTask);
            whereCond += this.AddTypesCondition(types);

            WorkItemCollection retVal = this.GetWorkItemCollection(String.Format(MainQuery, whereCond));
            var lst = new List<WorkItem>();
            if (retVal != null)
                lst.AddRange(retVal.Cast<WorkItem>());
            return lst;
        }

        public List<WorkItem> GetWorkItemListForUser(List<String> projects, List<WorkItemState> states, List<String> types, string userId, bool isTask = true)
        {
            // var whereCond = this.AddProjectsCondition(projects);
            // whereCond += String.Format(" AND [System.AssignedTo]='{0}'", userId);
            var whereCond = String.Format("([System.AssignedTo]='{0}')", userId);
            whereCond += this.AddStatesCondition(states, isTask);
            whereCond += this.AddTypesCondition(types);

            WorkItemCollection retVal = this.GetWorkItemCollection(String.Format(MainQuery, whereCond));
            var lst = new List<WorkItem>();
            if (retVal != null)
                lst.AddRange(retVal.Cast<WorkItem>());
            return lst;
        }

        public List<WorkItem> GetWorkItemList(string sQuery, IDictionary context)
        {
            WorkItemCollection retVal = this.GetWorkItemCollection(sQuery, context);
            var lst = new List<WorkItem>();
            if (retVal != null)
                lst.AddRange(retVal.Cast<WorkItem>());
            return lst;
        }

        private List<WorkItem> _cScore;

        public List<WorkItem> ScoredUserWorkItem
        {
            get { return _cScore; }
        }

        public void GetScoredUserWorkItemList(String userName, DateTime dScrored, IDictionary<string, string> projects)
        {
            String date = dScrored.ToShortDateString();
            String whereCond = this.AddProjectsCondition(projects);
            whereCond += String.Format(" AND (([System.WorkItemType]='Task' AND [System.State]='Closed' AND [Microsoft.VSTS.Common.ClosedDate]>='{0}' AND [Microsoft.VSTS.Common.Closedby]='{1}')", date, userName);
            whereCond += String.Format(" OR ([System.WorkItemType]='Bug' AND [System.State] in ('Resolved','Closed') AND [System.Reason] in ('Fixed','Verified') AND [Microsoft.VSTS.Common.ResolvedDate]>='{0}' AND [Microsoft.VSTS.Common.ResolvedBy]='{1}'))", date, userName);
            var retVal = this.GetWorkItemCollection(String.Format(MainQuery, whereCond));
            if (retVal != null)
            {
                this._cScore = new List<WorkItem>(retVal.Count);
                this._cScore.AddRange(retVal.Cast<WorkItem>());

            }
            else
            {
                this._cScore = null;
            }
        }

        public List<WorkItem> GetCompletedWorkItemList(DateTime dMin, Byte iType = 0)
        {
            return GetCompletedWorkItemList(dMin, null, iType);
        }

        public List<WorkItem> GetCompletedWorkItemList(DateTime dMin, String userName, Byte iType = 0)
        {
            String date = dMin.ToShortDateString();
            String typeName = iType == 3 ? "User Story" : "Task";
            String usConditions = iType == 3 ? " AND [System.Reason]='Acceptance tests pass'" : "";
            String taskCondition = !String.IsNullOrEmpty(userName)
                                       ? String.Format("([System.WorkItemType]='{2}' AND [Microsoft.VSTS.Common.ClosedDate]>='{0}' AND [Microsoft.VSTS.Common.Closedby]='{1}'{3})",
                                           date, userName, typeName, usConditions)
                                       : String.Format("([System.WorkItemType]='{1}' AND [Microsoft.VSTS.Common.ClosedDate]>='{0}'{2})",
                                           date, typeName, usConditions);
            String bugCondition = !String.IsNullOrEmpty(userName)
                                      ? String.Format("([System.WorkItemType]='Bug' AND [Microsoft.VSTS.Common.ResolvedDate]>='{0}'  AND [Microsoft.VSTS.Common.ResolvedBy]='{1}')",
                                          date, userName)
                                      : String.Format("([System.WorkItemType]='Bug' AND [Microsoft.VSTS.Common.ResolvedDate]>='{0}')",
                                          date);

            String whereCond = String.Empty;
            switch (iType)
            {
                case 0: // bugs
                    whereCond = bugCondition;
                    break;
                case 1://tasks
                case 3: // User story
                    whereCond = taskCondition;
                    break;
                case 2:// both
                    whereCond = bugCondition + " AND " + taskCondition;
                    break;
            }

            //whereCond += String.Format(" AND (([System.WorkItemType]='Task' AND [System.State]='Closed' AND [Microsoft.VSTS.Common.ClosedDate]>='{0}' AND [Microsoft.VSTS.Common.Closedby]='{1}')", date, userName);
            whereCond += this.AddStatesCondition(new List<WorkItemState> { WorkItemState.Done }, false);// String.Format(" ( [System.State] in ('Resolved','Closed')  AND [Microsoft.VSTS.Common.ResolvedDate]>='{0}')", date);
            var retVal = this.GetWorkItemCollection(String.Format(MainQuery, whereCond));
            var lst = new List<WorkItem>();
            if (retVal != null)
                lst.AddRange(retVal.Cast<WorkItem>());
            return lst;
        }

        public List<WorkItem> GetWorkItemListByCreated(DateTime dMin, List<string> types)
        {
            String date = dMin.ToShortDateString();
            String whereCond = String.Format("([System.CreatedDate]>='{0}')", date);
            whereCond += this.AddTypesCondition(types);
            var retVal = this.GetWorkItemCollection(String.Format(MainQuery, whereCond));
            var lst = new List<WorkItem>();
            if (retVal != null)
                lst.AddRange(retVal.Cast<WorkItem>());
            return lst;
        }

        public List<WorkItem> GetWorkItemListForUser(String userName, DateTime dMin, Nullable<DateTime> dMax = null)
        {
            String date = dMin.ToShortDateString();
            // who
            String whereCond = String.Format("([System.History] Contains '{0}' OR  [System.AssignedTo]='{0}')", userName);
            // String.Format("([System.ChangedBy]='{0}' OR [System.AssignedTo]='{0}' OR [Microsoft.VSTS.Common.ResolvedBy]='{0}' OR [Microsoft.VSTS.Common.ActivatedBy]='{0}' OR [Microsoft.VSTS.Common.ClosedBy]='{0}' OR [Microsoft.VSTS.Common.CreatedBy]='{0}')", userName);
            // when
            String[] dateConditions = { "[Changed Date]", "[Microsoft.VSTS.Common.StateChangeDate]", "[Microsoft.VSTS.Common.ResolvedDate]", "[Microsoft.VSTS.Common.ClosedDate]" };
            if (dMax == null)
            {
                whereCond += " AND (" + dateConditions.Aggregate("", (current, condition) => current + String.Format(" OR {0}>='{1}'", condition, date)).Substring(4) + ")";
                //whereCond += String.Format("AND ([Changed Date]>='{0}' OR [Microsoft.VSTS.Common.StateChangeDate]>='{0}' OR [Microsoft.VSTS.Common.ResolvedDate]>='{0}'  OR [Microsoft.VSTS.Common.ClosedDate]>='{0}')", date);
            }
            else
            {
                whereCond += " AND (" + dateConditions.Aggregate("", (current, condition) => current + String.Format(" OR ({0}>='{1}' AND {0}<='{2}')", condition, date, dMax.Value.ToShortDateString())).Substring(4) + ")";
                //       whereCond += String.Format(" AND ([Changed Date]>='{0}' AND [Changed Date]<='{1}')", date, dMax.Value.ToShortDateString());
            }
            var retVal = this.GetWorkItemCollection(String.Format(MainQuery, whereCond));
            var lst = new List<WorkItem>();
            if (retVal != null)
                lst.AddRange(retVal.Cast<WorkItem>());
            return lst;
        }

        public List<WorkItem> GetWorkItemListScheduled(List<String> projects, List<string> types, DateTime dStart, DateTime dEnd)
        {
            var whereCond = this.AddProjectsCondition(projects);
            whereCond += this.AddTypesCondition(types);
            whereCond += String.Format(" AND ( ([Microsoft.VSTS.Scheduling.StartDate]>='{0}' AND [Microsoft.VSTS.Scheduling.FinishDate]<'{1}')", dStart.Date, dEnd.Date.AddDays(2));
            whereCond += String.Format(" OR (([Microsoft.VSTS.Scheduling.StartDate]<'{0}' AND [Microsoft.VSTS.Scheduling.StartDate]!='')  {1})", dStart.Date,
                this.AddStatesCondition(new List<WorkItemState> { WorkItemState.Todo }, false));
            whereCond += String.Format(" OR ([Microsoft.VSTS.Scheduling.StartDate]<'{0}' {1}) )", dStart.Date,
                this.AddStatesCondition(new List<WorkItemState> { WorkItemState.InProgress, WorkItemState.Reopen, WorkItemState.ToTest }, false));

            WorkItemCollection retVal = this.GetWorkItemCollection(String.Format(MainQuery, whereCond));
            if (retVal != null)
            {
                var lst = new List<WorkItem>(retVal.Count);
                lst.AddRange(retVal.Cast<WorkItem>());
                return lst;
            }
            return null;
        }

        public List<WorkItem> GetWorkItemListUnScheduled(List<String> projects, List<WorkItemState> states, List<String> types)
        {
            var whereCond = this.AddProjectsCondition(projects);
            whereCond += this.AddTypesCondition(types);
            whereCond += this.AddStatesCondition(states, false);
            whereCond += " AND ([Microsoft.VSTS.Scheduling.StartDate]='' AND [Microsoft.VSTS.Scheduling.FinishDate]='')";
            whereCond += " AND [System.AssignedTo]=''";

            WorkItemCollection retVal = this.GetWorkItemCollection(String.Format(MainQuery, whereCond));
            if (retVal != null)
            {
                var lst = new List<WorkItem>(retVal.Count);
                lst.AddRange(retVal.Cast<WorkItem>());
                return lst;
            }
            return null;
        }

        public int GetUserScore()
        {
            return _cScore != null && _cScore.Count > 0 ? _cScore.Sum(p => p.GetScrumPilotScore()) : 0;
        }

        public decimal GetUserTimeCompleted()
        {
            return _cScore != null && _cScore.Count > 0 ? _cScore.Sum(p => p.GetCompletedWork()) : 0;
        }


        public WorkItemCollection GetWorkItemCollection(string sQuery)
        {
            try
            {
                var qry = new Query(_wis, sQuery);
                WorkItemCollection wic = qry.RunQuery();
                return wic;
            }
            catch (Exception)
            {
                return null;
            }
        }

        public WorkItemCollection GetWorkItemCollection(string sQuery, IDictionary context)
        {
            // todo : investigate to this side to improve some query 
            var qry = new Query(_wis, sQuery, context);
            WorkItemCollection wic = qry.RunQuery();
            return wic;
        }

        private String AddProjectsCondition(IEnumerable<string> projects)
        {
            var sRet = projects.Aggregate(String.Empty, (current, project) => current + (",'" + project + "'"));
            sRet = "[System.TeamProject] in (" + sRet.Substring(1) + ")";
            return sRet;
        }

        private String AddProjectsCondition(IEnumerable<KeyValuePair<string, string>> projects, Boolean bCurrentProject = false, Boolean bCurrentIteration = false)
        {
            return
                this.AddTuplesProjectsCondition(
                    projects.Select(project => new Tuple<string, string>(project.Key, project.Value)).ToArray(),
                    bCurrentProject, bCurrentIteration);
        }

        private String AddTuplesProjectsCondition(IEnumerable<Tuple<string, string>> projects, Boolean bCurrentProject = false, Boolean bCurrentIteration = false)
        {
            if (bCurrentProject)
            {
                var sRet = projects.Aggregate(String.Empty, (current, project) => current + (",'" + project.Item1 + "'"));
                if (sRet == "")
                    return sRet;
                sRet = "[System.TeamProject] in (" + sRet.Substring(1) + ")";
                return sRet;
            }
            else
            {
                String sCond = bCurrentIteration ? "OR ([System.TeamProject]='{0}' AND [System.IterationPath] ='{1}')" : "OR ([System.TeamProject]='{0}' AND [System.IterationPath] under '{1}')";
                // Give project condition with current iteration/sprint
                var sRet = projects.Aggregate(String.Empty, (current, project) => current + String.Format(sCond, project.Item1, project.Item2));
                if (sRet == "")
                    return sRet;
                sRet = "(" + sRet.Substring(3) + ")";
                return sRet;
            }
        }

        private String AddIdsCondition(IEnumerable<int> ids)
        {
            var sRet = ids.Aggregate(String.Empty, (current, id) => current + ("," + id.ToString(CultureInfo.InvariantCulture)));
            sRet = "[System.Id] in (" + sRet.Substring(1) + ")";
            return sRet;
        }

        private String AddStatesCondition(List<WorkItemState> states, Boolean isTask)
        {
            String sRet = String.Empty;
            if (states != null && states.Count > 0)
            {
                bool bOperande = false;
                sRet += " AND (";
                if (states.Exists(p => p == WorkItemState.Done))
                {
                    sRet += "([System.State] in ('Closed','Resolved','Termin','Ferm'))";
                    bOperande = true;
                }
                if (states.Exists(p => p == WorkItemState.InProgress))
                {
                    sRet += bOperande ? " OR " : "";
                    sRet += "(([System.State] in ('Actif','Active') AND [System.AssignedTo]<>'')";
                    sRet += " )";
                    bOperande = true;
                }
                if (states.Exists(p => p == WorkItemState.Todo))
                {
                    sRet += bOperande ? " OR " : "";
                    sRet += "(([System.State] in ('Actif','Active') AND [System.AssignedTo]='')";
                    if (isTask)
                        sRet += "OR ([System.State]='' AND [System.AssignedTo]='' AND [System.WorkItemType] in ('Task','Tche'))";
                    sRet += " )";
                    bOperande = true;
                }
                if (states.Exists(p => p == WorkItemState.Reopen))
                {
                    sRet += bOperande ? " OR " : "";
                    sRet += "([System.Reason] in ('Ractiv','Rgression','Reactivated','Regression','Not fixed','Test Failed'))";
                }
                else
                {
                    sRet += bOperande ? " AND " : "";
                    sRet += "([System.Reason] not in ('Ractiv','Rgression','Reactivated','Regression','Not fixed','Test Failed'))";
                }

                sRet += ")";
            }
            return sRet;
        }

        private String AddTypesCondition(List<String> types)
        {
            String sRet = String.Empty;
            if (types != null && types.Count > 0)
            {
                var tp = types.Aggregate(String.Empty, (current, type) => current + (",'" + type + "'"));
                sRet += " AND ([System.WorkItemType] in (" + tp.Substring(1) + "))";
            }
            return sRet;
        }



        #region Asynchronous Query

        //private Query _asyncQry;
        //public ICancelableAsyncResult BeginQuery(AsyncCallback callback, String wisql)
        //{
        //    this._asyncQry = new Query(_wis, wisql);
        //    ICancelableAsyncResult car = _asyncQry.BeginQuery();
        //    return car;
        //}

        //public ICollection<WorkItem> EndQuery(ICancelableAsyncResult car)
        //{
        //    try
        //    {
        //        var workItemCollection = this._asyncQry.EndQuery(car);
        //        var workItemList = new List<WorkItem>(workItemCollection.Count);

        //        foreach (WorkItem workItem in workItemCollection)
        //        {
        //            //  workItemList.Add(new WorkItem(workItem));
        //        }

        //        return workItemList;
        //    }
        //    catch (ConnectionException ce)
        //    {
        //        //throw new TfsConnectionException(ce);
        //    }
        //    return null;
        //}

        #endregion
    }
}