/*
COPYRIGHT (C) 2011-2013 Raynald Messi

THIS FILE IS PART OF SCRUMPILOT.

SCRUMPILOT IS FREE SOFTWARE: YOU CAN REDISTRIBUTE IT AND/OR MODIFY IT UNDER THE TERMS OF 
THE GNU LESSER GENERAL PUBLIC LICENSE VERSION v2.1 AS PUBLISHED BY THE FREE SOFTWARE FOUNDATION.

SCRUMPILOT IS DISTRIBUTED IN THE HOPE THAT IT WILL BE USEFUL, BUT WITHOUT ANY WARRANTY; WITHOUT
EVEN THE IMPLIED WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. SEE THE GNU LESSER
GENERAL PUBLIC LICENSE FOR MORE DETAILS.

YOU SHOULD HAVE RECEIVED A COPY OF THE GNU LESSER GENERAL PUBLIC LICENSE ALONG WITH SCRUMPILOT. 
IF NOT, SEE <HTTP://WWW.GNU.ORG/LICENSES/>.
*/

/// <reference path="Commons/Base.ts" />
/// <reference path="Commons/dashboard.d.ts" />
declare var Highcharts: any;

class KanbanDashBoard extends Base.Page {
    private bdgchart: any;

    Ready() {
        // prepare/compile template
        $.templates({
            KPITemplate: "#KPITemplate",
            ProjectsTemplate: "#ProjectsTemplate",
            KPIProjectsTemplate: "#KPIProjectsTemplate",
            PopOverTemplate: "#PopOverTemplate"
        });
        this.BugsList();
        this.KPIProjects();
        this.KanbanProject();
        $('html,.moreInfo').click(function (e) {
            $('.moreInfo').popover('hide');
        });
    }

    public BugsList(): void {
        $.getJSON(this.aUrl[0], null, function (data: KPIKanbanBugs) => {
            $("#keypointsdiv").html($.render.KPITemplate(data.kpi));

            this.InitGraph(data.BugItems.FixedBugs, data.BugItems.NewBugs, 'Fixed', 'New', 'bugGraph', 'Bugs History', 250, false);

            window.setTimeout(function ()=> {
                return this.BugsList();
            }, 240000);
        });
    }

    public KanbanProject(): void {
        var self = this;
        $.getJSON(self.aUrl[1], null, function (data: any) {
            $("#wilist").html($.render.ProjectsTemplate(data.Iterations));
            if (data.KanbanIpMax != -1)
                $("#kanbanipmax").empty().text('(' + data.KanbanIpMax + ')');
            if (data.KanbanTtMax != -1)
                $("#kanbanttmax").empty().text('(' + data.KanbanTtMax + ')');
            $(".moreInfo").popover(
            {
                html: true,
                placement: "left",
                title: "Items list",
                content: function () {
                    var element: any = $(this);
                    var div_id: string = "div-id-" + $.now();
                    $.getJSON(self.aUrl[3],
                     {
                         iteration: element.attr('data-id'),
                         state: element.attr('data-col')
                     }, function (data: any) {
                         $('#' + div_id).html($.render.PopOverTemplate(data));
                     });
                    return ('<ul class="sppop" id="' + div_id + '">Loading...</ul>')
                },
                trigger: 'manual'
            }).click(function (e) {
                $(this).popover('toggle');
                e.stopPropagation();
            });
            window.setTimeout(function () {
                return Base.Page.Self.KanbanProject();
            }, 300000);
        });
    }

    public KPIProjects(): void {
        $.getJSON(this.aUrl[2], null, function (data) => {
            $("#KPIProjects").html($.render.KPIProjectsTemplate(data.Totals));

            this.InitGraph(data.HistoryDone, null, null, null, 'gphTasksDone', 'Tasks done (by day)', 160, true);
            this.InitGraph(data.HistoryHoursCompleted, null, null, null, 'gphHoursDone', 'Hours done (by day)', 160, true);
            this.InitGraph(data.HistoryUsDone, null, null, null, 'gphUsDone', 'User Stories done (by day)', 160, true);

            window.setTimeout(function () => {
                return this.KPIProjects();
            }, 360000);
        });
    }

    private InitGraph(serie1: any, serie2: any, serie1Name: string, serie2Name: string, renderName: string, title: string, height: number, smallLabel: bool): void {
        var series: any[] = Array();
        if (serie1) {
            series.push({
                name: serie1Name,
                data: serie1
            });
        }
        if (serie2) {
            series.push({
                name: serie2Name,
                data: serie2
            });
        }
        if (series && series.length > 0) {
            var legend = series.length > 1;
            var xAxis: any = {
                categories: [
                            'D-6',
                            'D-5',
                            'D-4',
                            'D-3',
                            'D-2',
                            'D-1',
                            'Today'
                ],
                labels: {
                    rotation: -45,
                    align: 'right',
                }
            };
            var myTitle: any = {
                text: title
            };
            if (smallLabel) {
                xAxis.labels.style = {
                    fontSize: '8px'
                }
                myTitle.style = {
                    fontSize: '14px',
                }
            }
            this.bdgchart = new Highcharts.Chart({
                chart: {
                    renderTo: renderName,
                    height: height,
                    borderRadius: 0,
                    backgroundColor: '#FFF',
                    type: 'column'
                },
                legend: {
                    enabled: legend
                },
                credits: {
                    enabled: false
                },
                colors: [
                        '#00AEEF',
                        '#E23300'
                ],
                xAxis: xAxis,
                yAxis: {
                    min: 0,
                    title: {
                        text: null
                    },
                },
                title: myTitle,
                series: series
            });
        }
    };

}