/*
COPYRIGHT (C) 2011-2013 Raynald Messi

THIS FILE IS PART OF SCRUMPILOT.

SCRUMPILOT IS FREE SOFTWARE: YOU CAN REDISTRIBUTE IT AND/OR MODIFY IT UNDER THE TERMS OF 
THE GNU LESSER GENERAL PUBLIC LICENSE VERSION v2.1 AS PUBLISHED BY THE FREE SOFTWARE FOUNDATION.

SCRUMPILOT IS DISTRIBUTED IN THE HOPE THAT IT WILL BE USEFUL, BUT WITHOUT ANY WARRANTY; WITHOUT
EVEN THE IMPLIED WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. SEE THE GNU LESSER
GENERAL PUBLIC LICENSE FOR MORE DETAILS.

YOU SHOULD HAVE RECEIVED A COPY OF THE GNU LESSER GENERAL PUBLIC LICENSE ALONG WITH SCRUMPILOT. 
IF NOT, SEE <HTTP://WWW.GNU.ORG/LICENSES/>.
*/

/// <reference path="Commons/Base.ts" />
declare var Highcharts: any;

class UserView extends Base.Page {
    private bdgchart: any;

    Ready() {
        $.templates({
            WorkItemTemplate: "<a href='{{:Url}}' target='_blank' data-content='{{:Content}}'>{{:Title}}</a>&nbsp;&nbsp;",
            TimeLineTemplate: "#NTimeLineTemplate",
        });
        this.SetUserMainKPI();
        this.SetBugsHistory();
        this.SetTasksHistory();
        this.SetUserHistory();
        // init follow actions
        var _self = this;
        $('a[data-action]').click(function () {
            var action = $(this).attr('data-action');
            CallService('POST', _self.aUrl[4], { id: _self.options.userId, type: action }, true, function () { window.location.reload(); });
        });
    }

    private SetUserMainKPI(): void {
        $.getJSON(this.aUrl[0], { id: this.options.userId }, function (data: any) => {
            if (data) {
                this.SetWorkItemLine(data.UserStories, "#imgldw", "#wip");
                this.SetWorkItemLine(data.Tasks, "#imglts", "#lts");
                this.SetWorkItemLine(data.Bugs, "#imglbg", "#lbg");
                this.SetWorkItemLine(data.CheckOuts, "#imgldp", "#pcs");
                $("#kpiScore").html(data.Score);
                $("#kpiHours").html(data.Hours);
                $("#kpiBugs").html(data.Fixedbugs);
                $("#kpiTasks").html(data.TotalTasks);
            }
            window.setTimeout(function () => {
                return this.SetUserMainKPI();
            }, 240000);
        });

    }

    private SetBugsHistory(): void {
        $.getJSON(this.aUrl[1], { id: this.options.userId }, function (data: any) => {
            if (data) {
                this.InitGraph(data, null, null, null, 'grphBugs', 'Bugs History', 250, false);
                window.setTimeout(function () => {
                    return this.SetBugsHistory();
                }, 240000);
            }
        });
    }

    private SetTasksHistory(): void {
        $.getJSON(this.aUrl[2], { id: this.options.userId }, function (data: any) => {
            if (data && data.Item1 && data.Item2 && data.Item3) {
                this.InitGraph(data.Item1, null, null, null, 'grphHours', 'Completed Hours', 250, false);
                this.InitGraph(data.Item2, null, null, null, 'grphTasks', 'Done Tasks', 250, false);
                window.setTimeout(function () => {
                    return this.SetTasksHistory();
                }, 300000);
            }
        });
    }

    private SetUserHistory(): void {
        $.getJSON(this.aUrl[3], { id: this.options.userId }, function (data: any) => {
            if (data) {
                var lenght: number = data.length
                var dateArray: any[] = new Array();
                for (var index: number = 0; index < lenght; index++) {
                    var converDate = new Date(parseInt(data[index].Date.substr(6)));
                    data[index].Date = converDate;
                    data[index].SmallDate = converDate.toISOString();

                    var ISODate: string = converDate.toISOString().substr(0, 10)
                    var indexOfElem = this.arrayObjectIndexOf(dateArray, ISODate, "ISODate");
                    if (indexOfElem == -1) {
                        indexOfElem = dateArray.push({
                            ISODate: ISODate,
                             CompleteDate: converDate.toLocaleDateString(),
                            data: new Array()
                        }) - 1;
                    }
                    dateArray[indexOfElem].data.push(data[index])
                }

                var content: string = $.render.TimeLineTemplate(dateArray);
                $("#timeline-container").html(content);
                $('abbr.timeago').timeago();
                 window.setTimeout(function () => {
                    return this.SetUserHistory();
                }, 360000);
            }
        });
    }



    private SetWorkItemLine(data: any, imgName: string, elementName: string): void {
        var content: string = "<em>No work item</em>";
        if (data && data.length > 0) {
            content = $.render.WorkItemTemplate(data);
        }
        $(imgName).hide();
        $(elementName).empty().append(content);
    }

  

    private arrowPoints(containerName: string) {
        var s = $(containerName).find('.item');
        $.each(s, function (index, obj) {
            var myObj = $(obj);
            if (index % 2 == 1) {
                // set an small offset
                myObj.addClass('item-right');
            }
            var posLeft = myObj.css("left");
            var html = posLeft == "0px" ? 'rightCorner' : 'leftCorner';
            html = "<span class='" + html + "'></span>";
            myObj.prepend(html);
        });
    }

    private InitGraph(serie1: any, serie2: any, serie1Name: string, serie2Name: string, renderName: string, title: string, height: number, smallLabel: bool): void {
        var series: any[] = Array();
        if (serie1) {
            series.push({
                name: serie1Name,
                data: serie1
            });
        }
        if (serie2) {
            series.push({
                name: serie2Name,
                data: serie2
            });
        }
        if (series && series.length > 0) {
            var legend = series.length > 1;
            var xAxis: any = {
                categories: [
                            'D-6',
                            'D-5',
                            'D-4',
                            'D-3',
                            'D-2',
                            'D-1',
                            'Today'
                ],
                labels: {
                    rotation: -45,
                    align: 'right',
                }
            };
            var myTitle: any = {
                text: title
            };
            if (smallLabel) {
                xAxis.labels.style = {
                    fontSize: '8px'
                }
                myTitle.style = {
                    fontSize: '14px',
                }
            }
            this.bdgchart = new Highcharts.Chart({
                chart: {
                    renderTo: renderName,
                    height: height,
                    borderRadius: 0,
                    backgroundColor: '#FFF',
                    type: 'column'
                },
                legend: {
                    enabled: legend
                },
                credits: {
                    enabled: false
                },
                colors: [
                        '#1570A6',
                        '#E23300'
                ],
                xAxis: xAxis,
                yAxis: {
                    min: 0,
                    title: {
                        text: null
                    },
                },
                title: myTitle,
                series: series
            });
        }
    };

    /*
    
function followUser(url, id, action) {
    // action=0 : follow / 1:unfollow
    CallService('POST', url, { id: id, type: action }, true, function () { window.location.reload(); });
}

    */
}